"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var png_js_1 = __importDefault(require("png-js"));
var zlib_1 = __importDefault(require("zlib"));
exports.getPngImage = function (pdf, data) {
    var image = new png_js_1.default(data);
    var hasAlphaChannel = image.hasAlphaChannel;
    var pngBaseData = {
        Type: 'XObject',
        Subtype: 'Image',
        BitsPerComponent: hasAlphaChannel ? 8 : image.bits,
        Width: image.width,
        Height: image.height,
        Filter: 'FlateDecode',
    };
    if (!hasAlphaChannel) {
        var params = pdf.ref({
            Predictor: 15,
            Colors: image.colors,
            BitsPerComponent: image.bits,
            Columns: image.width,
        });
        pngBaseData['DecodeParms'] = params;
    }
    if (image.palette.length === 0) {
        pngBaseData['ColorSpace'] = image.colorSpace;
    }
    else {
        var palette = pdf.ref({
            stream: new Buffer(image.palette),
        });
        pngBaseData['ColorSpace'] = ['Indexed', 'DeviceRGB', image.palette.length / 3 - 1, palette];
    }
    if (image.transparency.grayscale != null) {
        var val = image.transparency.grayscale;
        pngBaseData['Mask'] = [val, val];
    }
    else if (image.transparency.rgb) {
        var rgb = image.transparency.rgb;
        var mask = [];
        for (var _i = 0, rgb_1 = rgb; _i < rgb_1.length; _i++) {
            var x = rgb_1[_i];
            mask.push(x, x);
        }
        pngBaseData['Mask'] = mask;
    }
    else if (image.transparency.indexed) {
        loadIndexedAlphaChannel(image);
    }
    else if (hasAlphaChannel) {
        splitAlphaChannel(pdf);
        var sMask = getSmask(pdf, image);
        pngBaseData['Mask'] = sMask;
    }
    var pngImage = pdf.ref(pngBaseData, undefined, image.imgData);
    return pngImage;
};
var loadIndexedAlphaChannel = function (image) {
    var transparency = image.transparency.indexed;
    return image.decodePixels(function (pixels) {
        var alphaChannel = new Buffer(image.width * image.height);
        var i = 0;
        for (var j = 0, end = pixels.length; j < end; j++) {
            alphaChannel[i++] = transparency[pixels[j]];
        }
        image.alphaChannel = zlib_1.default.deflateSync(alphaChannel);
    });
};
var splitAlphaChannel = function (image) {
    image.decodePixels(function (pixels) {
        var a, p;
        var colorCount = image.colors;
        var pixelCount = image.width * image.height;
        var imgData = new Buffer(pixelCount * colorCount);
        var alphaChannel = new Buffer(pixelCount);
        var i = (p = a = 0);
        var len = pixels.length;
        var skipByteCount = image.bits === 16 ? 1 : 0;
        while (i < len) {
            for (var colorIndex = 0; colorIndex < colorCount; colorIndex++) {
                imgData[p++] = pixels[i++];
                i += skipByteCount;
            }
            alphaChannel[a++] = pixels[i++];
            i += skipByteCount;
        }
        image.imgData = zlib_1.default.deflateSync(imgData);
        image.alphaChannel = zlib_1.default.deflateSync(alphaChannel);
    });
};
var getSmask = function (pdf, image) {
    var sMask;
    if (image.hasAlphaChannel) {
        sMask = pdf.ref({
            Type: 'XObject',
            Subtype: 'Image',
            Height: image.height,
            Width: image.width,
            BitsPerComponent: 8,
            Filter: 'FlateDecode',
            ColorSpace: 'DeviceGray',
            Decode: [0, 1],
            stream: image.alphaChannel,
        });
    }
    return sMask;
};
